/* VERSION.C */

/*
 *      less  (This Amiga version renamed 'ty')
 *      Copyright (c) 1984,1985  Mark Nudelman
 *
 *      This program may be freely used and/or modified, 
 *      with the following provisions:
 *      1. This notice and the above copyright notice must remain intact.
 *      2. Neither this program, nor any modification of it,
 *         may not be sold for profit without written consent of the author.
 *
 *      -----------------------------------------------------------------
 *
 *      This program is a paginator similar to "more", 
 *      but allows you to move both forward and backward in the file.  
 *      Commands are based on "more" and "vi".
 *
 *      ----------------------- CHANGES ---------------------------------
 *
 *          Allowed use on standard input               1/29/84   markn
 *          Added E, N, P commands                      2/1/84    markn
 *          Added '=' command, 'stop' signal handling   4/17/84   markn
 *          Added line folding                          4/20/84   markn
 *      v2: Fixed '=' command to use BOTTOM_PLUS_ONE, 
 *          instead of TOP, added 'p' & 'v' commands    4/27/84   markn
 *      v3: Added -m and -t options, '-' command        5/3/84    markn
 *      v4: Added LESS environment variable             5/3/84    markn
 *      v5: New comments, fixed '-' command slightly    5/3/84    markn
 *      v6: Added -Q, visual bell                       5/15/84   markn
 *      v7: Fixed jump_back(n) bug: n should count real
 *          lines, not folded lines.  Also allow number
 *          on G command.                               5/24/84   markn
 *      v8: Re-do -q and -Q commands                    5/30/84   markn
 *      v9: Added "+<cmd>" argument                     9/25/84   markn
 *      v10: Fixed bug in -b<n> argument processing     10/10/84  markn
 *      v11: Made error() ring bell if \n not entered.  10/18/84  markn
 *      -----------------------------------------------------------------
 *      v12: Reorganized signal handling and made
 *           portable to 4.2bsd.                        2/13/85   mark
 *      v13: Reword error message for '-' command.      2/16/85   mark
 *      v14: Added -bf and -bp variants of -b.          2/22/85   mark
 *      v15: Miscellaneous changes.                     2/25/85   mark
 *      v16: Added -u flag for backspace processing.    3/13/85   mark
 *      v17: Added j and k commands, 
 *              changed -t default.                     4/13/85   mark
 *      v18: Rewrote signal handling code.              4/20/85   mark
 *      v19: Got rid of "verbose" eq_message().         5/2/85    mark
 *           Made search() scroll in some cases.
 *      v20: Fixed screen.c ioctls for System V.        5/21/85   mark
 *      v21: Fixed some first_cmd bugs.                 5/23/85   mark
 *      v22: Added support for no RECOMP nor REGCMP.    5/24/85   mark
 *      v23: Miscellanous changes and prettying up.     5/25/85   mark
 *      v24: Added ti,te terminal init & de-init       6/3/85 Mike Kersenbrock
 *      v25: Added -U flag, standout mode underlining.  6/8/85    mark
 *      v26: Added -M flag.                             6/9/85    mark
 *           Use underline termcap (us) if it exists.
 *      v27: Renamed some variables to make unique in   6/15/85   mark
 *           6 chars.  Minor fix to -m.
 *      v28: Fixed right margin bug.                    6/28/85   mark
 *      v29: Incorporated M.Rose's changes to signal.c  6/28/85   mark
 *      v30: Fixed stupid bug in argument processing.   6/29/85   mark
 *      v31: Added -p flag, changed repaint algorithm.  7/15/85   mark
 *           Added kludge for magic cookie terminals.
 *      v32: Added cat_file if output not a tty.        7/16/85   mark
 *      v33: Added -e flag and EDITOR.                  7/23/85   mark
 *      v34: Added -s flag.                             7/26/85   mark
 *      v35: Rewrote option handling; added option.c.   7/27/85   mark
 *      v36: Fixed -e flag to work if not last file.    7/29/85   mark
 *      v37: Added -x flag.                             8/10/85   mark
 *      v38: Changed prompting; created prompt.c.       8/19/85   mark
 *      v39: (Not -p) does not initially clear screen.  8/24/85   mark
 *      v40: Added "skipping" indicator in forw().      8/26/85   mark
 *      v41: ONLY_RETURN, control char commands,        9/17/85   mark
 *           faster search, other minor fixes.
 *      v42: Added ++ command line syntax;              9/25/85   mark
 *           ch_fsize for pipes.
 *      v43: Added -h flag, changed prim.c algorithms.  10/15/85  mark
 *      v44: Made END print in all cases of eof;        10/16/85  mark
 *           ignore SIGTTOU after receiving SIGTSTP.
 *      v45: Never print backspaces unless -u.          10/16/85  mark
 *      v46: Backwards scroll in jump_loc.              10/24/85  mark
 *      v47: Fixed bug in edit(): *first_cmd==0         10/30/85  mark
 *      v48: Use TIOCSETN instead of TIOCSETP.          11/16/85  mark
 *           Added marks (m and ' commands).
 * ------------------------------------------------------------------------
 *    ty1.0: Major rehacks to suit Amiga.              15-DEC-86  ajw
 *           Modified for Lattice C compiler.
 *           Added workbench interface.
 *           Allowed use of arrow keys for scrolling.
 *    ty1.1: Spacebar to scroll page at a time         11-JUN-87  ajw
 *           Fixed bug in exit code that cause it to
 *           exit thru error routine on every exit.
 *    ty1.2: Modified output routines to allow thru    28-AUG-87  ajw
 *           ANSI escape sequences.
 *
 *    ty1.3: Major cosmetic destruction perpetrated    [33m7-Aug-89   JJB[m
 *           by JJB TEMPLAR. Version 1.3 will probably
 *           drop a lot of the features above.
 *    ty1.3: Opening Window, attaching console to it.  [33m8-Aug-89   JJB[m
 *           Use IDCMP for input, console for output.
 *           Two of the original modules have died :-(
 *                     signal.c
 *                     ttyin.c
 *           Several more modules were created :-)
 *                      win.c       ; window stuff
 *                      gadg.c      ; gadget stuff
 *                      image.c     ; image data (for CHIPMEM: lc -ad)
 *    ty1.3: Got gadgets and resize going. Continuing  [33m9-Aug-89   JJB[m
 *           rehack. Also got left/right going.
 *    ty1.3: Menu-button works with gadgets, cleaned   [33m10-Aug-89  JJB[m
 *           up most of display after resize.
 *    ty1.3: Added requester, search is back. Re-wrote [33m11-Aug 89  JJB[m
 *           help and about info (again). Repaint is
 *           back (using resize), plus a number of
 *           other minor modifications. Two new modules
 *           have been created :-)
 *                      req.c       ; requester stuff
 *                      search.c    ; ex-prim.c search code
 *           Noted "less" on Fish disks is incredibly easy
 *           to crash, and much bigger than ty. Put marks
 *           back, but use 0-9 instead of a-z because using
 *           RAWKEY instead of ANSI.
 *    ty1.3: Added "iconify", by Leo Shwab on Fish     [33m12-Aug 89  JJB[m
 *           Disk #126. Added icon based args.
 *           Renamed search.c to suppcom.c
 *    ty1.3: Added print, titlebar filename, and bool  [33m13-Aug 89  JJB[m
 *           function tacked on to string requester.
 *    ty1.3: Moved prompt to titlebar, added bottom    [33m14-Aug 89  JJB[m
 *           border gadgets. This means there is no
 *           spare line at the bottom, so you have to
 *           be more intelligent regarding where you
 *           put newlines (for put_line).
 *           Implemented Knuth-Morris-Pratt pattern
 *           matching algorithm. Still doesn't handle
 *           regular expressions. Added "default pattern"
 *           option. Also added ENV:ty check.
 *    ty1.3: Added def pattern option to ENV:ty.       [33m15-Aug 89  JJB[m
 *           Fixed silly bug in tup string (for search).
 *           Made horizontal bar operational. Needs some work.
 *           Made residentable. Piece of cake with cres.o!
 *           Works under WShell.
 *    ty1.3: Added mode changing.                      [33m16-Aug 89  JJB[m
 *           Zapped infinite loop bug in input.c. Turned
 *           out files that did not end with a '\n' character,
 *           were breaking ty in "fast_line on" mode.
 *    ty1.3: Tried WB-resident stuff intercepting      [33m21-Aug 89  JJB[m
 *           LoadSeg, etc, but failed. Wierd. Zapped a
 *           partition on my HD in the process, due to
 *           freaking out "blink".
 *    ty1.3: Added stuff for "click move".             [33m26-Aug 89  JJB[m
 *           Changed getcmd to use cmdmap arrays. This
 *           saved at least 600 bytes. Had hoped for more,
 *           but 600 is OK. Created cmdmap.c :-)
 *           Got sick of DisplayBeep, so added audio
 *           bell (from RKM & HRM). Created bell.c :-)
 *           Restructured screen.c stuff. Created screen.h :-)
 *           Also tinkered with option.c, to reduce size a bit
 *           more.
 *    ty1.3: Added '?' as wildcard for match().        [33m27-Aug 89  JJB[m
 *           Added newcli and editor functions. Created
 *           a new module, system.c :-)
 *           Added rudimentary paging. Forward oriented,
 *           only works properly if fast_line is off, and
 *           requires ^L to be first char on line (speed reasons).
 *           Pretty patchy paging plonked in. Goes forward OK,
 *           but backward looks horrible. This needs some work.
 *    ty1.3: Added command to load new file.           [33m28-Aug 89  JJB[m
 *    -----------------------------------------------------------------
 */

char version[] = "1st NZAmigaUG (Wgtn) Ty v1.3 - "__DATE__;
